#include <libplayerc++/playerc++.h>
#include <stdio.h>
#include <math.h>
#include <vector>

#include "cmdline_parsing.h"
#include "common_functions.h"
#include "occupancy_grid.h"

using namespace PlayerCc;
using namespace std;

int main(int argc, char **argv)
{
  	/* Calls the command line parser */
  	parse_args(argc, argv);

	double lower_left[2] = {-8, -8};
	double upper_right[2] = {8, 8};
	double cell_size = 0.05;

  	SimpleOccupancyGrid oc(lower_left, upper_right, cell_size);

  	try {
  		/* Initialize connection to player */
		PlayerClient robot(gHostname, gPort);
		Position2dProxy pp(&robot, gIndex);
		LaserProxy lp(&robot, gIndex); 

		int num_attempts = 20;
		if(!check_robot_connection(robot, pp, num_attempts)) 
			exit(-2);

		// Now we start the main processing loop
		int count = 0;
		while(1) {
			
			// read from the proxies; YOU MUST ALWAYS HAVE THIS LINE
			robot.Read();
			
			double pose[3] = { pp.GetXPos(), pp.GetYPos(), pp.GetYaw() };

			// these next three lines illustrate how you query the laserproxy to gather 
			// the laser scanner data        
			unsigned int n = lp.GetCount();
			
			
			if (n==0)
				// no data available
				continue;
			
			vector<double> range_data(n);
			vector<double> bearing_data(n);
			for(uint i=0; i<n; i++) {
				  range_data[i] = lp.GetRange(i);
				  bearing_data[i] = lp.GetBearing(i);
			}

			double max_range = 6;
			oc.addScan(pose, n, &bearing_data[0], &range_data[0], max_range);
			if (count % 20 == 0) {
				char filename[100];
				sprintf(filename, "oc-%05d.ppm", count);
				printf("Writing map on file %s.\n", filename);
				oc.savePPM(filename);
			}
			
			// Now laser range data can be accessed as a double vector, e.g. range_data[i] 
			// and bearing_data[i].


			// Implement your arc-based path planner here

			// Send the turnrate and speed to the robot
			double turnrate = 1;
			double speed = 0.2;
			pp.SetSpeed(speed, turnrate);
			
			count += 1;
		}

  	} catch(PlayerError e) {
  		write_error_details_and_exit(argv[0], e);
  	}

  	return 0;
}
